"""
Modify a py-postgresql connection to use rhythm datetimes instead of stdlib
datetimes.
"""
import postgresql.types
import postgresql.types.io.lib as iolib
from . import lib

def pack_date(date, pack = iolib.date_pack):
	return pack(date + 1)

def unpack_date(date, Date = lib.Date, unpack = iolib.date_unpack):
	return Date(unpack(date) - 1)

def pack_timestamp(timestamp, get_ms = lib.select.microsecond, pack = iolib.longlong_pack):
	return pack(timestamp.elapse(day=1) // 1000)

def unpack_timestamp(timestamp, Timestamp = lib.Timestamp, unpack = iolib.longlong_unpack):
	return Timestamp(unpack(timestamp) * 1000).rollback(day=1)

def pack_interval(measure, pack = iolib.qll_pack):
	# rhythm does not support composition of relative and absolute deltas
	ms = measure.select('microsecond', 'day')
	day = measure.select('day')
	return pack((ms, day, 0))

def unpack_interval(measure, Measure = lib.Measure.of, unpack = iolib.qll_unpack):
	# rhythm does not support composition of relative and absolute deltas
	ms, d, months = unpack(measure)
	return Measure(day = d, microsecond = ms)

def configure(connection):
	"""
	Configure a py-postgresql database connection to use rhythm datetimes.
	"""
	connection.typio._cache.update([
		(postgresql.types.DATEOID, (pack_date, unpack_date)),
		(postgresql.types.TIMESTAMPTZOID, (pack_timestamp, unpack_timestamp)),
		(postgresql.types.TIMESTAMPOID, (pack_timestamp, unpack_timestamp)),
		(postgresql.types.INTERVALOID, (pack_interval, unpack_interval)),
		#(postgresql.types.TIMEOID, ()),
		#(postgresql.types.TIMETZOID, ()),
	])
