import postgresql.temporal
from .. import libpostgres
from .. import lib

dates = (
	(lib.Date.of(iso='1982-05-18'), '1982-05-18'),
	(lib.Date.of(iso='2000-01-01'), '2000-01-01'),
	(lib.Date.of(iso='1000-01-01'), '1000-01-01'),
	(lib.Date.of(iso='500-01-01'), '500-01-01'),
)

timestamps = (
	(lib.Timestamp.of(iso='1982-05-18T2:40:05.5'), '1982-05-18T2:40:05.5'),
	(lib.Timestamp.of(iso='2000-01-01T0:0:0.0'), '2000-01-01T0:0:0.0'),
	(lib.Timestamp.of(iso='2000-01-01T0:0:-1'), '1999-12-31T23:59:59'),
	(lib.Timestamp.of(iso='1500-12-25T2:40:05.5'), '1500-12-25T2:40:05.5'),
)

intervals = (
	(lib.Measure.of(day=3, hour=2, minute=1, second=44), '3 days 2 hours 1 minute 44 seconds'),
	(lib.Measure.of(day=0, hour=2, minute=1, second=44), '2 hours 1 minute 44 seconds'),
	(lib.Measure.of(second=0), '0 seconds'),
)

def test_packing(test):
	'Validate that the packing functions are consistent before pushing the objects through PG.'
	for x, s in dates:
		test.fail_if_not_equal(libpostgres.unpack_date(libpostgres.pack_date(x)), x)

	for x, s in timestamps:
		test.fail_if_not_equal(libpostgres.unpack_timestamp(libpostgres.pack_timestamp(x)), x)

	for x, s in intervals:
		test.fail_if_not_equal(libpostgres.unpack_interval(libpostgres.pack_interval(x)), x)

@postgresql.temporal.pg_tmp
def test_postgres(test):
	'Validate that postgres interprets the objects as rhythm does.'

	# setup
	libpostgres.configure(db)
	db.settings['timezone'] = 'utc'

	dateio = prepare('select $1::date, $2::text::date = $1::date, ARRAY[$1::date::text, $2::text::date::text]').first
	timestampio = prepare('select $1::timestamp, $2::text::timestamp = $1, ARRAY[$1::timestamp::text, $2::text::timestamp::text]').first
	timestamptzio = prepare('select $1::timestamptz, $2::text::timestamptz = $1, ARRAY[$1::timestamptz::text, $2::text::timestamptz::text]').first
	intervalio = prepare('select $1::interval, $2::text::interval = $1, ARRAY[$1::interval::text, $2::text::interval::text]').first

	for x, s in dates:
		out, dbcmp, txt = dateio(x, s)
		test.fail_if_not_equal(out, x)
		test.fail_if_not(dbcmp)

	for x, s in timestamps:
		out, dbcmp, txt = timestampio(x, s)
		test.fail_if_not_equal(out, x)
		test.fail_if_not(dbcmp)

	for x, s in timestamps:
		out, dbcmp, txt = timestamptzio(x, s)
		test.fail_if_not_equal(out, x)
		test.fail_if_not(dbcmp)

	for x, s in intervals:
		out, dbcmp, txt = intervalio(x, s)
		test.fail_if_not_equal(out, x)
		test.fail_if_not(dbcmp)

if __name__ == '__main__':
	from dev import libtest; libtest.execmodule()
